/* pdflib.c -- Lua binding for PDFlib 4.0.3 */

#include <errno.h>
#include <string.h>

#include "lua.h"
#include "lauxlib.h"
#include "pdflib.h"

#define getstring	luaL_check_string
#define getostring(L,i)	luaL_opt_string(L,i,NULL)
#define getfloat	(float) luaL_check_number
#define getofloat(L,i)	(float) luaL_opt_number(L,i,0)
#define getint		(int) luaL_check_number
#define getlong		(long) luaL_check_number

static int TAG;

static int report(lua_State *L, int rc, const char *filename)
{
 if (rc==-1)
 {
  lua_pushnil(L);
  lua_pushstring(L,strerror(errno));
  lua_pushstring(L,filename);
  return 3;
 }
 else
 {
  lua_pushnumber(L,rc);
  return 1;
 }
}

static PDF *getpdf(lua_State *L, int i)
{
 if (lua_tag(L,i)!=TAG) luaL_argerror(L,i,"expected PDF handle");
 return lua_touserdata(L,i);
}

static void errorhandler(PDF *p, int t, const char *message)
{
 lua_State *L = PDF_get_opaque(p);
 PDF_delete(p);
 luaL_verror(L,"(PDFlib) %s",message);
}

static int L_PDF_shutdown(lua_State *L)
{
 PDF_shutdown();
 return 0;
}

static int L_PDF_new(lua_State *L)
{
 PDF *p = PDF_new2(errorhandler,NULL,NULL,NULL,L);
 lua_pushusertag(L,p,TAG);
 return 1;
}

static int L_PDF_delete(lua_State *L)
{
 PDF *p = getpdf(L,1);
 PDF_delete(p);
 return 0;
}

static int L_PDF_open_file(lua_State *L)
{
 PDF *p = getpdf(L,1);
 const char *filename = getostring(L,2);
 int rc = PDF_open_file(p,filename);
 return report(L,rc,filename);
}

static int L_PDF_close(lua_State *L)
{
 PDF *p = getpdf(L,1);
 PDF_close(p);
 return 0;
}

static int L_PDF_get_buffer(lua_State *L)
{
 PDF *p = getpdf(L,1);
 long size;
 const char *s = PDF_get_buffer(p,&size);
 lua_pushlstring(L,s,size);
 return 1;
}

static int L_PDF_begin_page(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float width = getfloat(L,2);
 float height = getfloat(L,3);
 PDF_begin_page(p,width,height);
 return 0;
}

static int L_PDF_end_page(lua_State *L)
{
 PDF *p = getpdf(L,1);
 PDF_end_page(p);
 return 0;
}

static int L_PDF_set_parameter(lua_State *L)
{
 PDF *p = getpdf(L,1);
 const char *key = getstring(L,2);
 const char *value = getstring(L,3);
 PDF_set_parameter(p,key,value);
 return 0;
}

static int L_PDF_set_value(lua_State *L)
{
 PDF *p = getpdf(L,1);
 const char *key = getstring(L,2);
 float value = getfloat(L,3);
 PDF_set_value(p,key,value);
 return 0;
}

static int L_PDF_get_parameter(lua_State *L)
{
 PDF *p = getpdf(L,1);
 const char *key = getstring(L,2);
 float modifier = getfloat(L,3);
 const char *value = PDF_get_parameter(p,key,modifier);
 lua_pushstring(L,value);
 return 1;
}

static int L_PDF_get_value(lua_State *L)
{
 PDF *p = getpdf(L,1);
 const char *key = getstring(L,2);
 float modifier = getofloat(L,3);
 float value = PDF_get_value(p,key,modifier);
 lua_pushnumber(L,value);
 return 1;
}

static int L_PDF_findfont(lua_State *L)
{
 PDF *p = getpdf(L,1);
 const char *fontname = getstring(L,2);
 const char *encoding = getstring(L,3);
 int embed = getint(L,4);
 int rc = PDF_findfont(p,fontname,encoding,embed);
 if (rc==-1)
  lua_pushnil(L);
 else
  lua_pushnumber(L,rc);
 return 1;
}

static int L_PDF_setfont(lua_State *L)
{
 PDF *p = getpdf(L,1);
 int font = getint(L,2);
 float fontsize = getfloat(L,3);
 PDF_setfont(p,font,fontsize);
 return 0;
}

static int L_PDF_show(lua_State *L)
{
 PDF *p = getpdf(L,1);
 const char *text = getstring(L,2);
 PDF_show2(p,text,lua_strlen(L,2));
 return 0;
}

static int L_PDF_show_xy(lua_State *L)
{
 PDF *p = getpdf(L,1);
 const char *text = getstring(L,2);
 float x = getfloat(L,3);
 float y = getfloat(L,4);
 PDF_show_xy2(p,text,lua_strlen(L,2),x,y);
 return 0;
}

static int L_PDF_continue_text(lua_State *L)
{
 PDF *p = getpdf(L,1);
 const char *text = getstring(L,2);
 PDF_continue_text2(p,text,lua_strlen(L,2));
 return 0;
}

static int L_PDF_show_boxed(lua_State *L)
{
 PDF *p = getpdf(L,1);
 const char *text = getstring(L,2);
 float left = getfloat(L,3);
 float top = getfloat(L,4);
 float width = getfloat(L,5);
 float height = getfloat(L,6);
 const char *hmode = getstring(L,7);
 const char *feature = getstring(L,8);
 int rc = PDF_show_boxed(p,text,left,top,width,height,hmode,feature);
 lua_pushnumber(L,rc);
 return 1;
}

static int L_PDF_set_text_pos(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float x = getfloat(L,2);
 float y = getfloat(L,3);
 PDF_set_text_pos(p,x,y);
 return 0;
}

static int L_PDF_stringwidth(lua_State *L)
{
 PDF *p = getpdf(L,1);
 const char *text = getstring(L,2);
 int font = getint(L,3);
 float size = getfloat(L,4);
 float width = PDF_stringwidth2(p,text,lua_strlen(L,2),font,size);
 lua_pushnumber(L,width);
 return 1;
}

static int L_PDF_setdash(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float b = getfloat(L,2);
 float w = getfloat(L,3);
 PDF_setdash(p,b,w);
 return 0;
}

#if 0
static int L_PDF_setpolydash(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float *dasharray = getfloat(L,2);
 int length = getint(L,3);
 PDF_setpolydash(p,dasharray,length);
 return 0;
}
#endif

static int L_PDF_setflat(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float flatness = getfloat(L,2);
 PDF_setflat(p,flatness);
 return 0;
}

static int L_PDF_setlinejoin(lua_State *L)
{
 PDF *p = getpdf(L,1);
 int linejoin = getint(L,2);
 PDF_setlinejoin(p,linejoin);
 return 0;
}

static int L_PDF_setlinecap(lua_State *L)
{
 PDF *p = getpdf(L,1);
 int linecap = getint(L,2);
 PDF_setlinecap(p,linecap);
 return 0;
}

static int L_PDF_setmiterlimit(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float miter = getfloat(L,2);
 PDF_setmiterlimit(p,miter);
 return 0;
}

static int L_PDF_setlinewidth(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float width = getfloat(L,2);
 PDF_setlinewidth(p,width);
 return 0;
}

static int L_PDF_initgraphics(lua_State *L)
{
 PDF *p = getpdf(L,1);
 PDF_initgraphics(p);
 return 0;
}

static int L_PDF_save(lua_State *L)
{
 PDF *p = getpdf(L,1);
 PDF_save(p);
 return 0;
}

static int L_PDF_restore(lua_State *L)
{
 PDF *p = getpdf(L,1);
 PDF_restore(p);
 return 0;
}

static int L_PDF_translate(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float tx = getfloat(L,2);
 float ty = getfloat(L,3);
 PDF_translate(p,tx,ty);
 return 0;
}

static int L_PDF_scale(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float sx = getfloat(L,2);
 float sy = getfloat(L,3);
 PDF_scale(p,sx,sy);
 return 0;
}

static int L_PDF_rotate(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float phi = getfloat(L,2);
 PDF_rotate(p,phi);
 return 0;
}

static int L_PDF_skew(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float alpha = getfloat(L,2);
 float beta = getfloat(L,3);
 PDF_skew(p,alpha,beta);
 return 0;
}

static int L_PDF_concat(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float a = getfloat(L,2);
 float b = getfloat(L,3);
 float c = getfloat(L,4);
 float d = getfloat(L,5);
 float e = getfloat(L,6);
 float f = getfloat(L,7);
 PDF_concat(p,a,b,c,d,e,f);
 return 0;
}

static int L_PDF_setmatrix(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float a = getfloat(L,2);
 float b = getfloat(L,3);
 float c = getfloat(L,4);
 float d = getfloat(L,5);
 float e = getfloat(L,6);
 float f = getfloat(L,7);
 PDF_setmatrix(p,a,b,c,d,e,f);
 return 0;
}

static int L_PDF_moveto(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float x = getfloat(L,2);
 float y = getfloat(L,3);
 PDF_moveto(p,x,y);
 return 0;
}

static int L_PDF_lineto(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float x = getfloat(L,2);
 float y = getfloat(L,3);
 PDF_lineto(p,x,y);
 return 0;
}

static int L_PDF_curveto(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float x1 = getfloat(L,2);
 float y1 = getfloat(L,3);
 float x2 = getfloat(L,4);
 float y2 = getfloat(L,5);
 float x3 = getfloat(L,6);
 float y3 = getfloat(L,7);
 PDF_curveto(p,x1,y1,x2,y2,x3,y3);
 return 0;
}

static int L_PDF_circle(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float x = getfloat(L,2);
 float y = getfloat(L,3);
 float r = getfloat(L,4);
 PDF_circle(p,x,y,r);
 return 0;
}

static int L_PDF_arc(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float x = getfloat(L,2);
 float y = getfloat(L,3);
 float r = getfloat(L,4);
 float alpha = getfloat(L,5);
 float beta = getfloat(L,6);
 PDF_arc(p,x,y,r,alpha,beta);
 return 0;
}

static int L_PDF_arcn(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float x = getfloat(L,2);
 float y = getfloat(L,3);
 float r = getfloat(L,4);
 float alpha = getfloat(L,5);
 float beta = getfloat(L,6);
 PDF_arcn(p,x,y,r,alpha,beta);
 return 0;
}

static int L_PDF_rect(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float x = getfloat(L,2);
 float y = getfloat(L,3);
 float width = getfloat(L,4);
 float height = getfloat(L,5);
 PDF_rect(p,x,y,width,height);
 return 0;
}

static int L_PDF_closepath(lua_State *L)
{
 PDF *p = getpdf(L,1);
 PDF_closepath(p);
 return 0;
}

static int L_PDF_stroke(lua_State *L)
{
 PDF *p = getpdf(L,1);
 PDF_stroke(p);
 return 0;
}

static int L_PDF_closepath_stroke(lua_State *L)
{
 PDF *p = getpdf(L,1);
 PDF_closepath_stroke(p);
 return 0;
}

static int L_PDF_fill(lua_State *L)
{
 PDF *p = getpdf(L,1);
 PDF_fill(p);
 return 0;
}

static int L_PDF_fill_stroke(lua_State *L)
{
 PDF *p = getpdf(L,1);
 PDF_fill_stroke(p);
 return 0;
}

static int L_PDF_closepath_fill_stroke(lua_State *L)
{
 PDF *p = getpdf(L,1);
 PDF_closepath_fill_stroke(p);
 return 0;
}

static int L_PDF_endpath(lua_State *L)
{
 PDF *p = getpdf(L,1);
 PDF_endpath(p);
 return 0;
}

static int L_PDF_clip(lua_State *L)
{
 PDF *p = getpdf(L,1);
 PDF_clip(p);
 return 0;
}

static int L_PDF_makespotcolor(lua_State *L)
{
 PDF *p = getpdf(L,1);
 const char *spotname = getstring(L,2);
 int len = getint(L,3);
 int rc = PDF_makespotcolor(p,spotname,len);
 lua_pushnumber(L,rc);
 return 1;
}

static int L_PDF_setcolor(lua_State *L)
{
 PDF *p = getpdf(L,1);
 const char *fstype = getstring(L,2);
 const char *colorspace = getstring(L,3);
 float c1 = getfloat(L,4);
 float c2 = getfloat(L,5);
 float c3 = getfloat(L,6);
 float c4 = getfloat(L,7);
 PDF_setcolor(p,fstype,colorspace,c1,c2,c3,c4);
 return 0;
}

static int L_PDF_begin_pattern(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float width = getfloat(L,2);
 float height = getfloat(L,3);
 float xstep = getfloat(L,4);
 float ystep = getfloat(L,5);
 int painttype = getint(L,6);
 int rc = PDF_begin_pattern(p,width,height,xstep,ystep,painttype);
 lua_pushnumber(L,rc);
 return 1;
}

static int L_PDF_end_pattern(lua_State *L)
{
 PDF *p = getpdf(L,1);
 PDF_end_pattern(p);
 return 0;
}

static int L_PDF_begin_template(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float width = getfloat(L,2);
 float height = getfloat(L,3);
 int rc = PDF_begin_template(p,width,height);
 lua_pushnumber(L,rc);
 return 1;
}

static int L_PDF_end_template(lua_State *L)
{
 PDF *p = getpdf(L,1);
 PDF_end_template(p);
 return 0;
}

static int L_PDF_place_image(lua_State *L)
{
 PDF *p = getpdf(L,1);
 int image = getint(L,2);
 float x = getfloat(L,3);
 float y = getfloat(L,4);
 float scale = getfloat(L,5);
 PDF_place_image(p,image,x,y,scale);
 return 0;
}

static int L_PDF_open_image(lua_State *L)
{
 PDF *p = getpdf(L,1);
 const char *imagetype = getstring(L,2);
 const char *source = getstring(L,3);
 const char *data = getstring(L,4);
 long length = getlong(L,5);
 int width = getint(L,6);
 int height = getint(L,7);
 int components = getint(L,8);
 int bpc = getint(L,9);
 const char *params = getstring(L,10);
 int rc = PDF_open_image(p,imagetype,source,data,length,width,height,components,bpc,params);
 lua_pushnumber(L,rc);
 return 1;
}

static int L_PDF_open_image_file(lua_State *L)
{
 PDF *p = getpdf(L,1);
 const char *imagetype = getstring(L,2);
 const char *filename = getstring(L,3);
 const char *stringparam = getstring(L,4);
 int intparam = getint(L,5);
 int rc = PDF_open_image_file(p,imagetype,filename,stringparam,intparam);
 return report(L,rc,filename);
}

static int L_PDF_close_image(lua_State *L)
{
 PDF *p = getpdf(L,1);
 int image = getint(L,2);
 PDF_close_image(p,image);
 return 0;
}

static int L_PDF_add_thumbnail(lua_State *L)
{
 PDF *p = getpdf(L,1);
 int image = getint(L,2);
 PDF_add_thumbnail(p,image);
 return 0;
}

static int L_PDF_open_CCITT(lua_State *L)
{
 PDF *p = getpdf(L,1);
 const char *filename = getstring(L,2);
 int width = getint(L,3);
 int height = getint(L,4);
 int BitReverse = getint(L,5);
 int K = getint(L,6);
 int BlackIs1 = getint(L,7);
 int rc = PDF_open_CCITT(p,filename,width,height,BitReverse,K,BlackIs1);
 return report(L,rc,filename);
}

static int L_PDF_add_bookmark(lua_State *L)
{
 PDF *p = getpdf(L,1);
 const char *text = getstring(L,2);
 int parent = getint(L,3);
 int open = getint(L,4);
 int rc = PDF_add_bookmark(p,text,parent,open);
 lua_pushnumber(L,rc);
 return 1;
}

static int L_PDF_set_info(lua_State *L)
{
 PDF *p = getpdf(L,1);
 const char *key = getstring(L,2);
 const char *value = getstring(L,3);
 PDF_set_info(p,key,value);
 return 0;
}

static int L_PDF_attach_file(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float llx = getfloat(L,2);
 float lly = getfloat(L,3);
 float urx = getfloat(L,4);
 float ury = getfloat(L,5);
 const char *filename = getstring(L,6);
 const char *description = getstring(L,7);
 const char *author = getstring(L,8);
 const char *mimetype = getstring(L,9);
 const char *icon = getstring(L,10);
 PDF_attach_file(p,llx,lly,urx,ury,filename,description,author,mimetype,icon);
 return 0;
}

static int L_PDF_add_note(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float llx = getfloat(L,2);
 float lly = getfloat(L,3);
 float urx = getfloat(L,4);
 float ury = getfloat(L,5);
 const char *contents = getstring(L,6);
 const char *title = getstring(L,7);
 const char *icon = getstring(L,8);
 int open = getint(L,9);
 PDF_add_note(p,llx,lly,urx,ury,contents,title,icon,open);
 return 0;
}

static int L_PDF_add_pdflink(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float llx = getfloat(L,2);
 float lly = getfloat(L,3);
 float urx = getfloat(L,4);
 float ury = getfloat(L,5);
 const char *filename = getstring(L,6);
 int page = getint(L,7);
 const char *dest = getstring(L,8);
 PDF_add_pdflink(p,llx,lly,urx,ury,filename,page,dest);
 return 0;
}

static int L_PDF_add_launchlink(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float llx = getfloat(L,2);
 float lly = getfloat(L,3);
 float urx = getfloat(L,4);
 float ury = getfloat(L,5);
 const char *filename = getstring(L,6);
 PDF_add_launchlink(p,llx,lly,urx,ury,filename);
 return 0;
}

static int L_PDF_add_locallink(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float llx = getfloat(L,2);
 float lly = getfloat(L,3);
 float urx = getfloat(L,4);
 float ury = getfloat(L,5);
 int page = getint(L,6);
 const char *dest = getstring(L,7);
 PDF_add_locallink(p,llx,lly,urx,ury,page,dest);
 return 0;
}

static int L_PDF_add_weblink(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float llx = getfloat(L,2);
 float lly = getfloat(L,3);
 float urx = getfloat(L,4);
 float ury = getfloat(L,5);
 const char *url = getstring(L,6);
 PDF_add_weblink(p,llx,lly,urx,ury,url);
 return 0;
}

static int L_PDF_set_border_style(lua_State *L)
{
 PDF *p = getpdf(L,1);
 const char *style = getstring(L,2);
 float width = getfloat(L,3);
 PDF_set_border_style(p,style,width);
 return 0;
}

static int L_PDF_set_border_color(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float red = getfloat(L,2);
 float green = getfloat(L,3);
 float blue = getfloat(L,4);
 PDF_set_border_color(p,red,green,blue);
 return 0;
}

static int L_PDF_set_border_dash(lua_State *L)
{
 PDF *p = getpdf(L,1);
 float b = getfloat(L,2);
 float w = getfloat(L,3);
 PDF_set_border_dash(p,b,w);
 return 0;
}

static int L_PDF_open_pdi(lua_State *L)
{
 PDF *p = getpdf(L,1);
 const char *filename = getstring(L,2);
 const char *stringparam = getstring(L,3);
 int intparam = getint(L,4);
 int rc = PDF_open_pdi(p,filename,stringparam,intparam);
 return report(L,rc,filename);
}

static int L_PDF_close_pdi(lua_State *L)
{
 PDF *p = getpdf(L,1);
 int doc = getint(L,2);
 PDF_close_pdi(p,doc);
 return 0;
}

static int L_PDF_open_pdi_page(lua_State *L)
{
 PDF *p = getpdf(L,1);
 int doc = getint(L,2);
 int page = getint(L,3);
 const char *label = getstring(L,4);
 int rc = PDF_open_pdi_page(p,doc,page,label);
 lua_pushnumber(L,rc);
 return 1;
}

static int L_PDF_place_pdi_page(lua_State *L)
{
 PDF *p = getpdf(L,1);
 int page = getint(L,2);
 float x = getfloat(L,3);
 float y = getfloat(L,4);
 float sx = getfloat(L,5);
 float sy = getfloat(L,6);
 PDF_place_pdi_page(p,page,x,y,sx,sy);
 return 0;
}

static int L_PDF_close_pdi_page(lua_State *L)
{
 PDF *p = getpdf(L,1);
 int page = getint(L,2);
 PDF_close_pdi_page(p,page);
 return 0;
}

static int L_PDF_get_pdi_parameter(lua_State *L)
{
 PDF *p = getpdf(L,1);
 const char *key = getstring(L,2);
 int doc = getint(L,3);
 int page = getint(L,4);
 int index = getint(L,5);
 int len;
 const char *s = PDF_get_pdi_parameter(p,key,doc,page,index,&len);
 lua_pushlstring(L,s,len);
 return 1;
}

static int L_PDF_get_pdi_value(lua_State *L)
{
 PDF *p = getpdf(L,1);
 const char *key = getstring(L,2);
 int doc = getint(L,3);
 int page = getint(L,4);
 int index = getint(L,5);
 int rc = PDF_get_pdi_value(p,key,doc,page,index);
 lua_pushnumber(L,rc);
 return 1;
}

#define Lua_register(L,s,f)

int lua_pdflibopen(lua_State *L)
{
 PDF_boot();
 TAG=lua_newtag(L);
 lua_pushcfunction(L,L_PDF_shutdown); lua_settagmethod(L,TAG,"gc");
 lua_pushusertag(L,NULL,TAG);
 TAG=lua_newtag(L);
 lua_pushcfunction(L,L_PDF_delete); lua_settagmethod(L,TAG,"gc");
 lua_register(L,"PDF_new",L_PDF_new);
 lua_register(L,"PDF_delete",L_PDF_delete);
 lua_register(L,"PDF_open_file",L_PDF_open_file);
 lua_register(L,"PDF_close",L_PDF_close);
 lua_register(L,"PDF_begin_page",L_PDF_begin_page);
 lua_register(L,"PDF_end_page",L_PDF_end_page);
 lua_register(L,"PDF_set_parameter",L_PDF_set_parameter);
 lua_register(L,"PDF_set_value",L_PDF_set_value);
 lua_register(L,"PDF_get_parameter",L_PDF_get_parameter);
 lua_register(L,"PDF_get_value",L_PDF_get_value);
 lua_register(L,"PDF_findfont",L_PDF_findfont);
 lua_register(L,"PDF_setfont",L_PDF_setfont);
 lua_register(L,"PDF_show",L_PDF_show);
 lua_register(L,"PDF_show_xy",L_PDF_show_xy);
 lua_register(L,"PDF_continue_text",L_PDF_continue_text);
 lua_register(L,"PDF_show_boxed",L_PDF_show_boxed);
 lua_register(L,"PDF_set_text_pos",L_PDF_set_text_pos);
 lua_register(L,"PDF_stringwidth",L_PDF_stringwidth);
 lua_register(L,"PDF_setdash",L_PDF_setdash);
 Lua_register(L,"PDF_setpolydash",L_PDF_setpolydash);
 lua_register(L,"PDF_setflat",L_PDF_setflat);
 lua_register(L,"PDF_setlinejoin",L_PDF_setlinejoin);
 lua_register(L,"PDF_setlinecap",L_PDF_setlinecap);
 lua_register(L,"PDF_setmiterlimit",L_PDF_setmiterlimit);
 lua_register(L,"PDF_setlinewidth",L_PDF_setlinewidth);
 lua_register(L,"PDF_initgraphics",L_PDF_initgraphics);
 lua_register(L,"PDF_save",L_PDF_save);
 lua_register(L,"PDF_restore",L_PDF_restore);
 lua_register(L,"PDF_translate",L_PDF_translate);
 lua_register(L,"PDF_scale",L_PDF_scale);
 lua_register(L,"PDF_rotate",L_PDF_rotate);
 lua_register(L,"PDF_skew",L_PDF_skew);
 lua_register(L,"PDF_concat",L_PDF_concat);
 lua_register(L,"PDF_setmatrix",L_PDF_setmatrix);
 lua_register(L,"PDF_moveto",L_PDF_moveto);
 lua_register(L,"PDF_lineto",L_PDF_lineto);
 lua_register(L,"PDF_curveto",L_PDF_curveto);
 lua_register(L,"PDF_circle",L_PDF_circle);
 lua_register(L,"PDF_arc",L_PDF_arc);
 lua_register(L,"PDF_arcn",L_PDF_arcn);
 lua_register(L,"PDF_rect",L_PDF_rect);
 lua_register(L,"PDF_closepath",L_PDF_closepath);
 lua_register(L,"PDF_stroke",L_PDF_stroke);
 lua_register(L,"PDF_closepath_stroke",L_PDF_closepath_stroke);
 lua_register(L,"PDF_fill",L_PDF_fill);
 lua_register(L,"PDF_fill_stroke",L_PDF_fill_stroke);
 lua_register(L,"PDF_closepath_fill_stroke",L_PDF_closepath_fill_stroke);
 lua_register(L,"PDF_endpath",L_PDF_endpath);
 lua_register(L,"PDF_clip",L_PDF_clip);
 lua_register(L,"PDF_makespotcolor",L_PDF_makespotcolor);
 lua_register(L,"PDF_setcolor",L_PDF_setcolor);
 lua_register(L,"PDF_begin_pattern",L_PDF_begin_pattern);
 lua_register(L,"PDF_end_pattern",L_PDF_end_pattern);
 lua_register(L,"PDF_begin_template",L_PDF_begin_template);
 lua_register(L,"PDF_end_template",L_PDF_end_template);
 lua_register(L,"PDF_place_image",L_PDF_place_image);
 lua_register(L,"PDF_open_image",L_PDF_open_image);
 lua_register(L,"PDF_open_image_file",L_PDF_open_image_file);
 lua_register(L,"PDF_close_image",L_PDF_close_image);
 lua_register(L,"PDF_add_thumbnail",L_PDF_add_thumbnail);
 lua_register(L,"PDF_open_CCITT",L_PDF_open_CCITT);
 lua_register(L,"PDF_add_bookmark",L_PDF_add_bookmark);
 lua_register(L,"PDF_set_info",L_PDF_set_info);
 lua_register(L,"PDF_attach_file",L_PDF_attach_file);
 lua_register(L,"PDF_add_note",L_PDF_add_note);
 lua_register(L,"PDF_add_pdflink",L_PDF_add_pdflink);
 lua_register(L,"PDF_add_launchlink",L_PDF_add_launchlink);
 lua_register(L,"PDF_add_locallink",L_PDF_add_locallink);
 lua_register(L,"PDF_add_weblink",L_PDF_add_weblink);
 lua_register(L,"PDF_set_border_style",L_PDF_set_border_style);
 lua_register(L,"PDF_set_border_color",L_PDF_set_border_color);
 lua_register(L,"PDF_set_border_dash",L_PDF_set_border_dash);
 lua_register(L,"PDF_open_pdi",L_PDF_open_pdi);
 lua_register(L,"PDF_close_pdi",L_PDF_close_pdi);
 lua_register(L,"PDF_open_pdi_page",L_PDF_open_pdi_page);
 lua_register(L,"PDF_place_pdi_page",L_PDF_place_pdi_page);
 lua_register(L,"PDF_close_pdi_page",L_PDF_close_pdi_page);
 lua_register(L,"PDF_get_pdi_parameter",L_PDF_get_pdi_parameter);
 lua_register(L,"PDF_get_pdi_value",L_PDF_get_pdi_value);
 lua_register(L,"PDF_get_buffer",L_PDF_get_buffer);
 return 1;
}
